

/*
 * THIS SOURCE CODE IS SUPPLIED  ``AS IS'' WITHOUT WARRANTY OF ANY KIND, 
 * AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH 
 * (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL 
 * WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND
 * ANY WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
 * RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR NOR
 * JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE FOR 
 * DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting the 
 * generality of the foregoing, neither the author, nor JAIR, nor JAIR's
 * publishers and distributors, warrant that the Source Code will be 
 * error-free, will operate without interruption, or will meet the needs 
 * of the user.
 */


/* 
 * C code for generating randomized bulldozer problems
 */





#include <stdlib.h>
#include <stdio.h>
#include <sys/timeb.h>






/* data structures ... (ha ha)
 */
typedef unsigned char Bool;
#define TRUE 1
#define FALSE 0



/* helpers
 */
void create_random_locations( void );
void print_random_origins( void );
void print_random_destins( void );



/* command line
 */
void usage( void );
Bool process_command_line( int argc, char *argv[] );




/* globals
 */

/* command line params
 */
int glocations, gpeople, gbulldozers;

/* random values
 */
int *gp_origin, *gb_origin, *gp_destin, *gb_destin;





int main( int argc, char *argv[] )

{

  int i;

  /* seed the random() function
   */
  struct timeb tp;
  ftime( &tp );
  srandom( tp.millitm );


  /* command line treatment, first preset values
   */
  glocations = -1;
  gpeople = -1;
  gbulldozers = -1;

  if ( argc == 1 || ( argc == 2 && *++argv[0] == '?' ) ) {
    usage();
    exit( 1 );
  }
  if ( !process_command_line( argc, argv ) ) {
    usage();
    exit( 1 );
  }

  create_random_locations();

  /* now output problem in PDDL syntax
   */
  printf("\n\n\n");

  /* header
   */
  printf("(define (problem bulldozer0-v%d-c%d)", 
	 gpeople, gbulldozers);
  printf("\n(:domain bulldozer)");

  printf("\n(:objects ");
  printf("\n          ");
  for ( i = 0; i < glocations; i++ ) printf("l%d ", i);
  printf("- place");
  printf("\n          ");
  for ( i = 0; i < gpeople; i++ ) printf("p%d ", i);
  printf("\n          ");
  for ( i = 0; i < gbulldozers; i++ ) printf("b%d ", i);
  printf("- thing)");

  printf("\n(:init");
  for ( i = 0; i < glocations - 1; i++ ) {
    printf("\n(road l%d l%d) (road l%d l%d)", i, i+1, i+1, i);
  }
  printf("\n(bridge l%d l%d) (bridge l%d l%d)", i, 0, 0, i);
  for ( i = 0; i < gpeople; i++ ) {
    printf("\n(mobile p%d)", i);
    printf("\n(person p%d)", i);
  }
  for ( i = 0; i < gbulldozers; i++ ) {
    printf("\n(vehicle b%d)", i);
    printf("\n(not-mobile b%d)", i);
  }
  print_random_origins();
  printf("\n)");

  printf("\n(:goal");
  printf("\n(and");
  print_random_destins();
  printf("\n)");
  printf("\n)");

  printf("\n)");

  printf("\n\n\n");

  exit( 0 );

}
  
  









/* random problem generation functions
 */







void create_random_locations( void )

{

  int r, i;

  gp_origin = ( int * ) calloc( gpeople, sizeof( int ) );
  gp_destin = ( int * ) calloc( gpeople, sizeof( int ) );
  gb_origin = ( int * ) calloc( gbulldozers, sizeof( int ) );
  gb_destin = ( int * ) calloc( gbulldozers, sizeof( int ) );

  for ( i = 0; i < gpeople; i++ ) {
    r = random() % glocations;
    gp_origin[i] = r;
  }
  for ( i = 0; i < gpeople; i++ ) {
    r = random() % glocations;
    gp_destin[i] = r;
  }
  for ( i = 0; i < gbulldozers; i++ ) {
    r = random() % glocations;
    gb_origin[i] = r;
  }
  for ( i = 0; i < gbulldozers; i++ ) {
    r = random() % glocations;
    gb_destin[i] = r;
  }

}







/* printing fns
 */


  




void print_random_origins( void )

{

  int i;

  for ( i = 0; i < gpeople; i++ ) {
    printf("\n(at p%d l%d)", i, gp_origin[i] );
  }
  for ( i = 0; i < gbulldozers; i++ ) {
    printf("\n(at b%d l%d)", i, gb_origin[i] );
  }

}



void print_random_destins( void )

{

  int i;

  for ( i = 0; i < gpeople; i++ ) {
    printf("\n(at p%d l%d)", i, gp_destin[i] );
  }
  for ( i = 0; i < gbulldozers; i++ ) {
    printf("\n(at b%d l%d)", i, gb_destin[i] );
  }

}




/* command line functions
 */










void usage( void )

{

  printf("\nusage:\n");

  printf("\nOPTIONS   DESCRIPTIONS\n\n");
  printf("-l <num>    number of locations (minimal 2)\n");
  printf("-p <num>    number of people (minimal 1)\n");
  printf("-b <num>    number of bulldozers (minimal 1)\n\n");

}



Bool process_command_line( int argc, char *argv[] )

{

  char option;

  while ( --argc && ++argv ) {
    if ( *argv[0] != '-' || strlen(*argv) != 2 ) {
      return FALSE;
    }
    option = *++argv[0];
    switch ( option ) {
    default:
      if ( --argc && ++argv ) {
	switch ( option ) {
	case 'l':
	  sscanf( *argv, "%d", &glocations );
	  break;
	case 'p':
	  sscanf( *argv, "%d", &gpeople );
	  break;
	case 'b':
	  sscanf( *argv, "%d", &gbulldozers );
	  break;
	default:
	  printf( "\n\nunknown option: %c entered\n\n", option );
	  return FALSE;
	}
      } else {
	return FALSE;
      }
    }
  }

  if ( glocations < 2 ||
       gpeople < 1 ||
       gbulldozers < 1 ) {
    return FALSE;
  }

  return TRUE;

}

